@@ -1,36 +1,41 @@
 #!/usr/bin/env python
 # coding=utf-8
-from ConfigParser import ConfigParser
 from .rpc_client import rpc_client
 from . import __version__
+from urlparse import urlparse
 import feedparser
-import urlparse
 import argparse
 import hashlib
 import urllib
 import redis
+import types
 import json
+import yaml
 import time
 import sys
 import os
 
-CONF_FILE = '~/.trmaid.conf'
+CONF_FILE = '~/.trmaid.yaml'
 CONF_FILE_FULL = os.path.expanduser(CONF_FILE)
 
 
 def msg_and_exit(*msg):
-    print '\n'.join(msg)
+    print '\n'.join(map(str, msg))
     sys.exit(1)
 
 
-def utfme(text):
-    if isinstance(text, unicode):
-        text = text.encode('utf-8')
-    return text
-
-
-def url_hostname(url):
-    return urlparse.urlparse(url).hostname
+def utfme(elem):
+    if isinstance(elem, types.DictType):
+        for key in elem.keys():
+            elem[key] = utfme(elem[key])
+    elif isinstance(elem, (types.ListType, types.TupleType)):
+        res = []
+        for e in elem:
+            res.append(utfme(e))
+        elem = res
+    elif isinstance(elem, basestring):
+        elem = elem.encode('utf-8')
+    return elem
 
 
 def normalize_dir(directory):
@@ -50,27 +55,23 @@ class TRMaid(object):
             )
 
         # config
-        self.conf = ConfigParser()
-        self.conf.read(config_file)
-        self.sections = self.conf.sections()
+        self.conf = yaml.load(open(CONF_FILE_FULL))
+        general = self.conf.get('general', {})
+        transmission = self.conf.get('transmission', {})
 
         # verbose
-        self.verbose = verbose
-        if self.conf.has_option('host', 'verbose'):
-            self.verbose = (
-                self.verbose or self.conf.getboolean('host', 'verbose')
-            )
+        self.verbose = verbose or general.get('verbose', False)
 
         self.msg('using config from %s' % config_file)
 
         # redis
-        if (self.conf.has_section('host') and
-            self.conf.has_option('host', 'redis')
-            ):
-            rds_url = self.conf.get('host', 'redis')
+        if 'redis' in general:
+            rds_url = general['redis']
         else:
             rds_url = 'redis://localhost:6379'
 
+        self.msg('redis url is %s' % rds_url)
+
         try:
             self.rds = redis.from_url(rds_url)
             self.rds.ping()
@@ -78,22 +79,20 @@ class TRMaid(object):
             msg_and_exit("Please install and run redis on %s" % rds_url)
 
         # transmission conf check
-        if not self.conf.has_option('host', 'url'):
-            msg_and_exit('host.url not configured')
-        elif not self.conf.has_option('host', 'username'):
-            msg_and_exit('host.username not configured')
-        elif not self.conf.has_option('host', 'password'):
-            msg_and_exit('host.password not configured')
+        if 'url' not in transmission:
+            msg_and_exit('transmission.url not configured')
+        elif 'username' not in transmission:
+            msg_and_exit('transmission.username not configured')
+        elif 'password' not in transmission:
+            msg_and_exit('transmission.password not configured')
 
         # rpc stuff
-        verify = True
-        if self.conf.has_option('host', 'verify'):
-            verify = self.conf.getboolean('host', 'verify')
+        verify = general.get('verify', True)
 
         self.rpc = rpc_client(
-            self.conf.get('host', 'url'),
-            self.conf.get('host', 'username'),
-            self.conf.get('host', 'password'),
+            transmission['url'],
+            transmission['username'],
+            transmission['password'],
             verify=verify
         )
 
@@ -110,29 +109,25 @@ class TRMaid(object):
         return result
 
     def conf_get(self, section, item, cast=None):
-        if self.conf.has_option(section, item):
-            if cast == bool:
-                func = self.conf.getboolean
-            elif cast == int:
-                func = self.conf.getint
-            elif cast == float:
-                func = self.conf.getfloat
-            else:
-                func = self.conf.get
-            return func(section, item)
+        if section in self.conf and item in self.conf[section]:
+            return self.conf[section][item]
 
     def handle_session(self):
-        if not self.conf.has_section('session'):
+        if 'session' not in self.conf:
             return self.msg('No [session] section')
 
         req = {}
         args = self.rpc.session_get().get('arguments', {})
+
         free = args.get('download-dir-free-space', 0)
         if free:
-            units = float(args.get('units', {}).get('size-bytes', 1000))
+            units = float(1024)
             free_gb = free / (units * units * units)
+            free = args.get('download-dir-free-space', 0)
             if free_gb < 50:
-                self.msg('only %.2f Gb free space left!' % free_gb)
+                self.msg('ONLY %.2F GB FREE SPACE LEFT!!' % free_gb)
+            else:
+                self.msg('%.2f Gb free space left' % free_gb)
 
         options = [
             # Booleans
@@ -190,9 +185,10 @@ class TRMaid(object):
                 "script-torrent-done-filename",
             ])
         ]
+        session = self.conf['session']
         for cast, keys in options:
             for key in keys:
-                value = self.conf_get('session', key, cast=cast)
+                value = session.get(key)
                 if value is None:
                     continue
                 current = args.get(key)
@@ -200,20 +196,16 @@ class TRMaid(object):
                     req[key] = value
 
         # blocklist
-        blocklist_enabled = self.conf_get(
-            'session', 'blocklist-enabled', cast=bool
-        )
-        blocklist_update_interval = self.conf_get(
-            'session', 'blocklist-update-interval', cast=int
-        )
-        blocklist_url = self.conf_get('session', 'blocklist-url')
+        blocklist_enabled = session.get('blocklist-enabled')
+        blocklist_update_interval = session.get('blocklist-update-interval')
+        blocklist_url = session.get('blocklist-url')
 
-        if blocklist_update_interval and blocklist_enabled and blocklist_url:
+        if blocklist_enabled and blocklist_update_interval and blocklist_url:
             if self.rds.setnx('trmaid:blocklist-update', '1'):
                 self.rds.expire(
                     'trmaid:blocklist-update', blocklist_update_interval * 3600
                 )
-                self.verbose_func(self.rpc.blocklist_update)
+                # self.verbose_func(self.rpc.blocklist_update)
 
         if req:
             self.verbose_func(self.rpc.session_set, req)
@@ -247,36 +239,40 @@ class TRMaid(object):
             'uploadRatio',
         ]})
         if torrents.get('result') == 'success':
-            torrents = torrents.get('arguments', {}).get('torrents', [])
+            torrents = utfme(torrents.get('arguments', {}).get('torrents', []))
             self.msg('there are %d total torrents...' % len(torrents))
             for tor in torrents:
                 tor['status_text'] = self.rpc.get_status(tor['status'])
                 tor['percentDone'] *= 100.0
-                tor['name'] = utfme(tor['name'])
                 self.handle_torrent(tor)
 
     def handle_torrent(self, tor):
+        sections = self.conf.get('tracker', [])
         for tracker in tor['trackers']:
             announce = tracker.get('announce')
             if not announce:
                 continue
-            hostname = url_hostname(announce)
-            if hostname not in self.sections:
+            hostname = urlparse(announce).hostname
+            if hostname not in sections:
                 continue
-            self.handle_torrent_section(tor, hostname)
+            self.handle_torrent_section(tor, 'tracker', hostname)
             return
 
-        location_section = 'location:%s' % normalize_dir(tor['downloadDir'])
-        if self.conf.has_section(location_section):
-            self.handle_torrent_section(tor, location_section)
+        sections = self.conf.get('location', [])
+        location = normalize_dir(tor['downloadDir'])
+        if location in sections:
+            self.handle_torrent_section(tor, 'location', location)
+            return
 
-        elif 'general-tracker' in self.sections:
-            self.handle_torrent_section(tor, 'general-tracker')
+        sections = self.conf.get('general', [])
+        if 'general' in sections:
+            self.handle_torrent_section(tor, 'tracker', 'general')
 
-    def handle_torrent_section(self, tor, section):
+    def handle_torrent_section(self, tor, section, subsection):
+        conf = self.conf[section][subsection]
         for key in ['download-dir', 'location']:
-            if self.conf.has_option(section, key):
-                correct = normalize_dir(self.conf.get(section, key))
+            if key in conf:
+                correct = normalize_dir(conf.get(key))
                 current = normalize_dir(tor['downloadDir'])
                 if current != correct:
                     self.verbose_func(
@@ -288,11 +284,12 @@ class TRMaid(object):
             stopped = tor['status_text'] == 'Stopped'
 
             for hkey in ['remove-stopped-after-hours', 'remove-after-hours']:
-                if self.conf.has_option(section, hkey):
+                if hkey in conf:
                     if not stopped and hkey == 'remove-stopped-after-hours':
                         remove.append(0)
                         break
-                    limit = self.conf.getfloat(section, hkey)
+                    limit = float(conf[hkey])
+                    print 'limit', limit
                     has_been = time.time() - tor['doneDate']
                     if has_been > limit * 3600:
                         remove.append(1)
@@ -301,11 +298,11 @@ class TRMaid(object):
                         break
 
             for rkey in ['remove-stopped-after-ratio', 'remove-after-ratio']:
-                if self.conf.has_option(section, rkey):
+                if rkey in conf:
                     if not stopped and rkey == 'remove-stopped-after-ratio':
                         remove.append(0)
                         break
-                    limit = self.conf.getfloat(section, rkey)
+                    limit = float(conf[rkey])
                     if tor['uploadRatio'] >= limit:
                         remove.append(1)
                     else:
@@ -317,87 +314,76 @@ class TRMaid(object):
                 self.verbose_func(self.rpc.torrent_remove, tor['id'])
                 return
 
-        req = self.get_torrent_set_args(section, tor)
+        req = self.get_torrent_set_args(section, subsection, tor)
         if req:
             req['ids'] = tor['id']
             self.verbose_func(self.rpc.torrent_set, req)
 
-    def get_torrent_add_args(self, section):
+    def get_torrent_add_args(self, section, subsection):
         req = {}
-        options = [
+        keys = [
             # Booleans
-            (bool, [
-                "paused",
-            ]),
+            "paused",
             # Doubles / Floats
-            (float, [
-                "seedRatioLimit",
-            ]),
+            "seedRatioLimit",
             # Integers
-            (int, [
-                "peer-limit",
-                "bandwidthPriority",
-            ]),
+            "peer-limit",
+            "bandwidthPriority",
             # Text / Strings
-            (None, [
-                "cookies",
-                "download-dir",
-                "metainfo",
-            ])
+            "cookies",
+            "download-dir",
+            "metainfo",
         ]
-        for cast, keys in options:
-            for key in keys:
-                value = self.conf_get(section, key, cast=cast)
-                if value is None:
-                    continue
+        for key in keys:
+            value = self.conf.get(section, {}).get(subsection, {}).get(key)
+            if value is None:
+                continue
+            req[key] = value
+
+        return req
+
+    def get_torrent_set_args(self, section, subsection, tor=None):
+        req = {}
+        keys = [
+            # Booleans
+            "downloadLimited",
+            "honorsSessionLimits",
+            "uploadLimited",
+            # Doubles / Floats
+            "seedRatioLimit",
+            # Integers
+            "bandwidthPriority",
+            "downloadLimit",
+            "peer-limit",
+            "queuePosition",
+            "seedIdleLimit",
+            "seedIdleMode",
+            "seedRatioMode",
+            "uploadLimit",
+        ]
+        conf = self.conf.get(section, {}).get(subsection, {})
+        for key in keys:
+            value = conf.get(key)
+            if value is None:
+                continue
+            if tor is None or tor[key] != value:
                 req[key] = value
 
-        return req
-
-    def get_torrent_set_args(self, section, tor=None):
-        req = {}
-        options = [
-            # Booleans
-            (bool, [
-                "downloadLimited",
-                "honorsSessionLimits",
-                "uploadLimited",
-            ]),
-            # Doubles / Floats
-            (float, [
-                'seedRatioLimit',
-            ]),
-            # Integers
-            (int, [
-                "bandwidthPriority",
-                "downloadLimit",
-                "peer-limit",
-                "queuePosition",
-                "seedIdleLimit",
-                "seedIdleMode",
-                "seedRatioMode",
-                "uploadLimit",
-            ]),
-        ]
-        for cast, keys in options:
-            for key in keys:
-                value = self.conf_get(section, key, cast=cast)
-                if value is None:
-                    continue
-                if tor is None or tor[key] != value:
-                    req[key] = value
-
-        if self.conf.has_option(section, 'location'):
-            location = self.conf.get(section, 'location')
+        if conf.get('location'):
+            location = conf['location']
             if tor is None or tor['downloadDir'] != location:
                 req['location'] = location
+
+        if 'trackers' in conf:
+            req['trackerAdd'] = conf['trackers']
+
         return req
 
-    def add_links(self, section, links, rss=False):
+    def add_links(self, section, subsection, links, rss=False):
         ids = []
-        rdskey = 'trmaid:added:%s' % hashlib.md5(section).hexdigest()
+        rdskey = 'trmaid:added:%s' % hashlib.md5(subsection).hexdigest()
         radded = rss and self.rds.lrange(rdskey, 0, -1) or []
-        add_req = self.get_torrent_add_args(section)
+        add_req = self.get_torrent_add_args(section, subsection)
 
         for link in links:
             xhash = hashlib.md5(link).hexdigest()
@@ -421,46 +407,38 @@ class TRMaid(object):
         if ids:
             if rss:
                 self.rds.ltrim(rdskey, 0, 1000)
-            set_req = self.get_torrent_set_args(section)
+            set_req = self.get_torrent_set_args(section, subsection)
             set_req['ids'] = ids
             res = self.verbose_func(self.rpc.torrent_set, set_req)
 
-    def handle_rss_and_redis_queues(self):
-        for section in self.sections:
-            if section.startswith('rss:'):
-                self.handle_rss_section(section)
-            elif section.startswith('redis:'):
-                self.handle_redis_section(section)
-
-    def handle_redis_section(self, section):
-        queue = section[6:]
-        self.msg('handling queue %s...' % queue[:40])
-
-        def links():
+    def handle_redis(self):
+        def links(queue):
             while 1:
                 link = self.rds.lpop(queue)
                 if not link:
                     break
                 yield link
-        return self.add_links(section, links())
 
-    def handle_rss_section(self, section):
-        feed_url = section[4:]
-        self.msg('handling feed %s...' % feed_url[:40])
+        for queue in self.conf.get('redis', {}).keys():
+            self.msg('handling queue %s...' % queue[:40])
+            self.add_links('redis', queue, links(queue))
 
-        def links():
+    def handle_rss(self):
+        def links(feed_url):
             for entry in feedparser.parse(feed_url).entries:
                 if not entry.link:
                     continue
                 yield urllib.unquote(entry.link)
 
-        return self.add_links(section, links(), rss=1)
+        for feed_url in self.conf.get('rss', {}).keys():
+            self.msg('handling feed %s...' % feed_url[:40])
+            self.add_links('rss', feed_url, links(feed_url), rss=True)
 
     def run(self):
         self.handle_session()
+        self.handle_rss()
+        self.handle_redis()
         self.handle_all_torrents()
-        self.handle_rss_and_redis_queues()
-
 
 def main():
     parser = argparse.ArgumentParser()
